#define DATA_HEADER 10001
#define SETTINGS_HEADER 10002
#define GPS_HEADER 10003

#define RXD2 16  // RX from ardusimple
#define TXD2 17 // TX to ardusimple
#define DEBUG Serial
#define RTK Serial1

/// UDP Variables
WiFiUDP Udp;  // Creation of wifi Udp instance
const char *ssid = {"YOURSSIDHERE"};

char buff[1025];
char GNGGA[1025];
char GNVTG[1025];

char packetBuffer[255];
uint16_t openGradePort = 9999; //OpenGrade Server Port
uint16_t gradeControlPort = 7777; // GradeControl  Port


IPAddress openGradeIP(192,168,1,0);   // what we want the sp 32 IPAddress to be
IPAddress gradeControlIP(192,168,1,255);   // Declaration of default IP for server 
IPAddress gatewayIP(192,168,1,1);   // 
IPAddress Subnet(255, 255, 255, 0);
IPAddress Dns(8,8,8,8);

IPAddress senderIP(0,0,0,0);
uint16_t senderPort;


void setup() {
  // put your setup code here, to run once:
  SetupUdp();

}

void loop() {
  // put your main code here, to run repeatedly:
  RecvGPSDAta();
  RecvUdpData();
}



void RecvGPSDAta(){
    
  if(RTK.available()){   
    
    int size = RTK.readBytesUntil('\n', buff, sizeof(buff));  
    for(int h = 0; h < size; h++) 
    {
      GNVTG[h] = buff[h]; 
    }
    memset(buff, 0, sizeof(buff)); // clear buff   
    
    int size2 = RTK.readBytesUntil('\n', buff, sizeof(buff));
    for(int h = 0; h < size2; h++) 
    {
      GNGGA[h] = buff[h];    
      
    }
    memset(buff, 0, sizeof(buff)); // clear buff 

    //SENDING
    SendGPSOverUDP();
    
  }
}


void SendGPSOverUDP(){
  
  Udp.beginPacket(openGradeIP,openGradePort);   //Initiate transmission of data
  Udp.print(gpsHeader);
  Udp.print(",");
  Udp.print(GNGGA);
  Udp.print(GNVTG);
  Udp.print("\r\n"); // End segment    
  Udp.endPacket();  // Close communication
  
  memset(buff, 0, sizeof(buff));// clear buff
  memset(GNGGA, 0, sizeof(GNGGA));// clear buff
  memset(GNGGA, 0, sizeof(GNGGA)); // clear buff

}

bool SetupUdp(){  
  
  
  WiFi.mode(WIFI_AP);
  WiFi.softAP(ssid);
  delay(100);
  
  WiFi.softAPConfig(gradeControlIP, gatewayIP, Subnet);  

  DEBUG.println(ssid);
  DEBUG.println(WiFi.softAPIP());
  
  Udp.begin(gradeControlIP, gradeControlPort);
  
  DEBUG.print(openGradeIP);
  DEBUG.print(" PORT->");
  DEBUG.println(openGradePort);
  DEBUG.print(gradeControlIP);
  DEBUG.print(" PORT->");
  DEBUG.println(gradeControlPort);
  
  return true;
}

bool SendUdpData(const char * _dataType)
{  
  //SENDING
  Udp.beginPacket(openGradeIP,openGradePort);   //Initiate transmission of data
  Udp.print(_dataType);
  Udp.print(",");
  Udp.print(b_autoState);
  Udp.print(",");    
  Udp.print(voltage);
  Udp.print(",");
  Udp.print(headingIMU);
  Udp.print(",");
  Udp.print(pitchIMU);
  Udp.print(",");
  Udp.print(rollIMU);      
  Udp.print("\r\n");   // End segment    
  Udp.endPacket();  // Close communication

  return true;
}

bool RecvUdpData()
{ 
  //DEBUG.print("RECV Func Called-> ");
  char *strings[20];
  char *ptr = NULL;  

  //RECEPTION
  int packetSize = Udp.parsePacket();   // Size of packet to receive
  
  senderIP = Udp.remoteIP();  //Sent from IP
  senderPort = Udp.remotePort();  //Sent from port

  if (packetSize) {       // If we received a package    
    
    Udp.read(packetBuffer, sizeof(packetBuffer));  
    
    byte index = 0;
    ptr = strtok(packetBuffer, ",");  // takes a list of delimiters    
    
    while(ptr != NULL)
    {
      strings[index] = ptr;      
      index++;
      ptr = strtok(NULL, ",");  // takes a list of delimiters
    }
    
    for(int n = 0; n < index; n++)
    { 
      OG_data[n] = strings[n];        
    }
    
    // convert to int to read couldnt read *PTR for some rsn   
    header = atoi(OG_data[0]);         
      
    switch (header)
    {
      case DATA_HEADER:
        DEBUG.println("DATA");
        b_deltaDir =  atoi(OG_data[1]);   // Cut Delta Dir
        b_autoState =  atoi(OG_data[2]);    // Cut Delta 
        b_cutDelta =  atoi(OG_data[3]);   // Auto State
        return true;
      break;


      case SETTINGS_HEADER:
        DEBUG.println("SETTINGS FOUND!");      
        b_Kp = atoi(OG_data[1]);
        b_Ki = atoi(OG_data[2]);
        b_Kd = atoi(OG_data[3]);
        b_retDeadband = atoi(OG_data[4]);
        b_extDeadband = atoi(OG_data[5]);
        b_valveType = atoi(OG_data[6]);
        Kp = double(b_Kp);
        Ki = double(b_Ki / 100);
        Kd = double(b_Kp * 100);  
        SetValveLimits();
        return true;
      break;

      default:
      break;
      
    }  
    return true;
  }
  return false;
}
